% timeAdvance.m
% u[1] ->  u, bacterial cell concentration
% u[2] ->  v, chemoattractant concentration

function u = timeAdvance(x,y,d2,d1x,d1y,bc,bi,st,stBC)

alpha = 2.25;          %  continuous rings
%  alpha = 5.0;        %  spotted rings  
du = 0.25;  
beta = 0.2; 
delta = 20; 
rho = 0.01; 
mu = 1;                                
dt = 0.005;  
finalTime = 75.0;  
N = length(x);
ns = length(st(1,:));   %  stencil size 
np = length(bi);        %  number of boundary points
ss = size(bc);  
u = zeros(N,2);
I = find( sqrt(x.^2 + y.^2) <= 0.5 ); 
u(I,1) = 1.0; 
U = u;
td = delaunay(x,y); t = 0; n = 0;
        
while t<finalTime
  u = rk4(U,dt,@F);
  v = u;
  
  u(1:np,1) = dot(bc, reshape(v(stBC(:,2:end),1),ss(1),ss(2)) ,2);
  u(1:np,2) = dot(bc, reshape(v(stBC(:,2:end),2),ss(1),ss(2)) ,2);   
  
  t = t + dt; 
  n = n + 1; 
  U = u;
  
  P = 200;
  if rem(n,P) == 0   % plot the solution ever P timesteps 
    h = trisurf(td,x,y,U(1:N,1));                
     xlabel('x'), ylabel('y'), title(sprintf('t = %6.4f',n*dt)), drawnow
  end                  
end

trisurf(td,x,y,U(1:N,1));  

% -----------------------------------

function fp = F(V)
  v = V;                      % no flux BCs
  V(1:np,1) = dot(bc, reshape(v(stBC(:,2:end),1),ss(1),ss(2)) ,2);  
  V(1:np,2) = dot(bc, reshape(v(stBC(:,2:end),2),ss(1),ss(2)) ,2);
      
  fp = zeros(N,2);    
  fp(:,1) =   du*( d2*V(:,1)  ) ...
    - alpha*(( d2*V(:,2)  ).*(V(:,1)./(1+V(:,2)).^2) + ...
     (d1x*V(:,2)).*( d1x*(V(:,1)./(1+V(:,2)).^2)) ) ...
    - alpha*( (d1y*V(:,2)).*(  d1y*(V(:,1)./(1+V(:,2)).^2)) ) + ...
      rho*V(:,1).*(delta - V(:,1));              
  fp(:,2) = d2*V(:,2) + beta*V(:,1).^2 - V(:,1).*V(:,2);
end

function W = rk4(w,k,F)  %  4th order Runge-Kutta 
 s1 = feval(F,w);   s2 = feval(F,w + k*s1/2); s3 = feval(F,w + k*s2/2); 
 s4 = feval(F,w + k*s3);  W = w + k*(s1 + 2*s2 + 2*s3 + s4)/6;  
end

end
