function [d2,d1x,d1y,bc] = weights(st,stBC,x,y,c,minK,maxK,dc,bi,R)                               
% INPUTS
%     st  indexes of the stencil centers
%   x, y  center coordinates
%      c  initial shape parameter
%   minK, maxK  lower\upper bound for systme matrix condition number
%     dc  shape parameter increment, e.g. 0.02
%     bi  indices of boundary centers  (assumed to be in position 1:length(bi))
%      R  radius of the circular domain
% OUTPUTS
%     d2, d1x, d1y  weights to discretize    u_xx + u_yy, u_x, and u_y
%     bc    weights for the zero flux boundary conditions                          

warning off
N = length(x);   
M = length(bi);            %  total and boundary centers
n = length(st(1,:));  
o = ones(1,n); 

d2 = sparse(N,n);
d1x = sparse(N,n);
d1y = sparse(N,n);

Rf = 1.0/R;
   
for i=1:N                    %  interior centers 
  pn = st(i,:); 
  rx = x(pn)*o - (x(pn)*o)';
  ry = y(pn)*o - (y(pn)*o)';
  r = sqrt( rx.^2 + ry.^2 ); 
  
  K = 1;
  while (K<minK | K>maxK)
    B = phi(r,c);    
    [U,S,V] = svd(B); 
    K = S(1,1)/S(n,n);
    if K<minK,  c = c - dc;
    elseif K>maxK, c = c + dc; end   
  end  
         
  Bi = V*diag(1./diag(S))*U';
  h = (D2( sqrt( (x(i)- x(pn)).^2 + (y(i)-y(pn)).^2 ),x(i)- x(pn),c) + ... 
      D2( sqrt( (x(i)- x(pn)).^2 + (y(i)-y(pn)).^2 ),y(i)- y(pn),c))';        
  d2(i,pn) = h*Bi;
  
  h = D1( sqrt( (x(i)- x(pn)).^2 + (y(i)-y(pn)).^2 ),x(i)- x(pn),c)';
  d1x(i,pn) = h*Bi; 

  h =  D1( sqrt( (x(i)- x(pn)).^2 + (y(i)-y(pn)).^2 ),y(i)- y(pn),c)';
  d1y(i,pn) = h*Bi; 

  
  if i<=M             %  weights for enforcing no flux BCs
    pn = stBC(i,:);  
    ob = ones(1,length(pn));
    rx = x(pn)*ob-(x(pn)*ob)'; 
    ry = y(pn)*ob-(y(pn)*ob)';
     r = sqrt(rx.^2+ry.^2);
    
    
    B = phi(r,c); [U,S,V] = svd(B); Bi = V*diag(1./diag(S))*U';
    h = D1( sqrt( (x(i)- x(pn)).^2 + (y(i)-y(pn)).^2 ),x(i)- x(pn),c)';
    d1xB = h*Bi; 
    
    h = D1( sqrt( (x(i)- x(pn)).^2 + (y(i)-y(pn)).^2 ),y(i)- y(pn),c)';
    d1yB = h*Bi; 
    bc(i,:)=(-Rf*x(i)*d1xB(2:end)-Rf*y(i)*d1yB(2:end))/(Rf*x(i)*d1xB(1) + ...
            Rf*y(i)*d1yB(1));

  end      
end  

warning on


     
% ------ MQ RBF and first and second derivatives ------------------------------
     
 function v = phi(r,s), v = sqrt(1 + (r.*s).^2);
 function df = D1(r,rh,s),  df  = (rh.*(s^2))./sqrt(1.0 + (r.*s).^2);
 function df = D2(r,rh,s)
    df  = (s^2)./sqrt(1.0+(r.*s).^2)-((rh.^2)*(s^4))./(1.0+(r.*s).^2).^(1.5);
